#version 330
#pragma optionNV (unroll all)

/*
 * Adapted from http://kayru.org/articles/dssdo/
 */

in vec2 TexCoord0;
in vec2 XYPos0; // = TexCoord0 * 2 - 1

out vec4 FragColor;

uniform sampler2D NormalTexture; // normal texture from GBuffer
uniform sampler2D DepthTexture; // depth texture from GBuffer
uniform sampler2D NoiseTexture;

uniform vec3 EyePosition_world;
uniform mat4 InvViewProjMatrix;

uniform vec2  RenderTargetSize;
uniform float RadiusOcclusion;
uniform float MaxDistanceOcclusion;


// TODO: WIR BRAUCHEN EINEN RAND von 50 pixel auf jeder Seite (oben, unten, links, rechts) den wir im GBuffer extra rendern mssen, aber dann weghauen

const vec3 POINTS[32] = vec3[] (
	vec3(-0.134, 0.044, -0.825),
	vec3(0.045, -0.431, -0.529),
	vec3(-0.537, 0.195, -0.371),
	vec3(0.525, -0.397, 0.713),
	vec3(0.895, 0.302, 0.139),
	vec3(-0.613, -0.408, -0.141),
	vec3(0.307, 0.822, 0.169),
	vec3(-0.819, 0.037, -0.388),
	vec3(0.376, 0.009, 0.193),
	vec3(-0.006, -0.103, -0.035),

	vec3(0.098, 0.393, 0.019),
	vec3(0.542, -0.218, -0.593),
	vec3(0.526, -0.183, 0.424),
	vec3(-0.529, -0.178, 0.684),
	vec3(0.066, -0.657, -0.570),
	vec3(-0.214, 0.288, 0.188),
	vec3(-0.689, -0.222, -0.192),
	vec3(-0.008, -0.212, -0.721),
	vec3(0.053, -0.863, 0.054),
	vec3(0.639, -0.558, 0.289),

	vec3(-0.255, 0.958, 0.099),
	vec3(-0.488, 0.473, -0.381),
	vec3(-0.592, -0.332, 0.137),
	vec3(0.080, 0.756, -0.494),
	vec3(-0.638, 0.319, 0.686),
	vec3(-0.663, 0.230, -0.634),
	vec3(0.235, -0.547, 0.664),
	vec3(0.164, -0.710, 0.086),
	vec3(-0.009, 0.493, -0.038),
	vec3(-0.322, 0.147, -0.105),

	vec3(-0.554, -0.725, 0.289),
	vec3(0.534, 0.157, -0.250)
);

const int NUMBER_POINTS = 16;
const vec2 NOISE_TEXTURE_SIZE = vec2(4, 4);

vec3 positionFromDepth(in vec2 texCoord, in vec2 xyCoord)
{
	float z = texture(DepthTexture, texCoord).r * 2 - 1; // range between [-1, 1]
	vec4 projectedPos = vec4(xyCoord, z, 1.0f);

	vec4 pos_world_hom = InvViewProjMatrix * projectedPos;
	return pos_world_hom.xyz / pos_world_hom.w;
}

vec3 positionFromDepth(in vec2 texCoord)
{
	return positionFromDepth(texCoord, texCoord * 2 - 1);
}

void main()
{
	vec3 position_world = positionFromDepth(TexCoord0, XYPos0);

	vec3  centerPos  = position_world;
	vec3 centerNormal = texture(NormalTexture, TexCoord0).xyz;
	float centerDepth  = distance(EyePosition_world, centerPos);

	float radius = RadiusOcclusion / centerDepth;
	float attenuationAngleThreshold = 0.1; // ~ cos(84)

	vec3 noise = texture(NoiseTexture, TexCoord0 * RenderTargetSize.xy / NOISE_TEXTURE_SIZE).xyz * 2 - 1;

	// fudge factors (= correction factors)
	const float correctionFactorLevel0 = 2.0;
	const float correctionFactorLevel1 = 10.0;

	// Spherical Harmonic weights:
	// https://en.wikipedia.org/wiki/Spherical_harmonics#List_of_spherical_harmonics
	const float sh2_weight_l0 = correctionFactorLevel0 * 0.28209; // 0.5*sqrt(1.0/pi);
	const vec3 sh2_weight_l1 = vec3(correctionFactorLevel1 * 0.48860); // 0.5*sqrt(3.0/pi);

	const vec4 sh2_weight = vec4(sh2_weight_l1, sh2_weight_l0) / NUMBER_POINTS;

	vec4 occlusion_sh2 = vec4(0.0);
	for(int i = 0; i < NUMBER_POINTS; i++)
	{
	    vec2 texOffset = reflect( POINTS[i].xy, noise.xy ).xy * radius;
		vec3 samplePos = positionFromDepth(TexCoord0 + texOffset);

		vec3 centerToSample = samplePos - centerPos;
		float distanceToCenter = length(centerToSample);

		vec3 centerToSampleNormalized = centerToSample / distanceToCenter;
		float attenuation = 1 - clamp(distanceToCenter / MaxDistanceOcclusion, 0.0, 1.0);
		float dp = dot(centerNormal, centerToSampleNormalized);

		attenuation = attenuation * attenuation * step(attenuationAngleThreshold, dp); // step: 0.0 if dp is smaller than threshold and otherwise 1.0

		occlusion_sh2 += attenuation * sh2_weight * vec4(centerToSampleNormalized, 1);
	}

	FragColor = occlusion_sh2;
}
