#version 330

in vec2 TexCoord0;
flat in vec4 Velocity0;
flat in float TTL0;
flat in uint Type0;
flat in vec3 Color0;

uniform sampler2D Texture1;
uniform sampler2D Texture2;
uniform sampler2D Texture3;
uniform sampler2D Texture4;
uniform sampler2D Texture5;

uniform float CurrentTime;

out vec4 FragColor;

vec3 pow3(vec3 v, float p)
{
	return pow(abs(v), vec3(p));
}

vec3 pulsatingSphere(vec3 color, vec2 p, float radius)
{
	// Inspired by: https://www.shadertoy.com/view/XdfXWS
	const float pulseSpeed = 12.0;
	const vec3 black = vec3(0.0);

	float value = fract( CurrentTime * pulseSpeed ) * 2.0 + 1.0; // between [1,3]	
	if (value < 2.0) // value between [1,2]
	{
		// increasing:
		return mix( black, vec3(1.0,1.0,0.5), value*(1.0-smoothstep( 0.0, radius, length(p))));
	}
	else // value between [2,3]
	{
		// decreasing:
		value = 4.0 - value;
		return mix( black, vec3(1.0,1.0,0.5), value*(1.0-smoothstep( 0.0, radius, length(p))));
	}
}

vec3 glowSphere(vec3 color, vec2 p, float radius, float glow)
{
	// Inspired by https://www.shadertoy.com/view/4lfXRf
	float term = 1./length(p)/5.;
	return pow3(vec3(color.x * term * 0.5, color.y * term * 0.5, color.z * term * 0.5), glow) * radius;
}

void main()
{
	if (Type0 == 0u) // Flare (start particle)
	{
		vec3 baseColor = vec3(1, 1, 0.66); // 255, 255, 170
		vec3 color = pulsatingSphere(baseColor, TexCoord0 * 2 - 1, 1);
		float lum = dot(color, vec3(0.299, 0.587, 0.114));
		FragColor = vec4(color, lum);
	}
	else if (Type0 == 1u) // Flare, after explosion
	{
		const float radius = 15;
		const float glow = 2.25; // how quickly the glow should decrease (higher -> quicker decrease)
		
		float mixT = clamp(TTL0, 0.0, 1.0);
		//vec3 baseColor = vec3(0.588, 1.0, 0.588); // 150, 255, 150 (green)
		vec3 color = glowSphere(Color0, TexCoord0 * 2 - 1, radius, glow);
		float lum = dot(color, vec3(0.299, 0.587, 0.114));
		FragColor = vec4(color, 1) * lum * mixT;
	}
	else if (Type0 == 2u) // Trail 1
	{
		float lum = texture(Texture1, TexCoord0).r * 2;

		const vec3 colorStart = vec3(0.922, 0.941, 1); // 235, 240, 255 ~= 7500 K
		const vec3 colorEnd   = vec3(1, 0.588, 0); // 255, 150, 0 ~= 2500 K
		float mixT = clamp(TTL0, 0.0, 1.0);
		float mixC = clamp(TTL0 * 0.5, 0.0, 1.0);

		vec3 color = mix(colorEnd, colorStart, mixC) * lum * mixT;
		FragColor = vec4(color, lum);
	}
	else if (Type0 == 3u) // Trail 2
	{
		// unused
	}
	else if (Type0 == 4u) // Trail 3
	{
		// unused
	}
	else if (Type0 == 5u) // Trail 4
	{
		// unused
	}
	else
	{
		// unused
	}
}