#version 330

// Adapted from: http://www.geeks3d.com/20140815/particle-billboarding-with-the-geometry-shader-glsl/

layout (points) in;
layout (triangle_strip, max_vertices = 4) out;

uniform mat4 ModelMatrix;   
uniform mat4 ModelViewMatrix;
uniform mat4 ProjectionMatrix;

uniform float CurrentTime;

in vData
{
	vec4 Velocity1;
	uint Type1;
	float TTL1;
	vec3 Color1;
} vertex[];

out vec2 TexCoord0;
flat out vec4 Velocity0;
flat out uint Type0;
flat out float TTL0;
flat out vec3 Color0;

/*
// quad will be rotated according to velocity vector
void emitRotatedQuad()
{
	vec4 P = gl_in[0].gl_Position;
	vec4 velocity = vertex[0].Velocity1;
	vec3 direction = normalize(velocity.xyz);
	vec3 right = vec3(1, 0, 0);
	vec3 some = cross(right, direction);
	right = cross(direction, some);

	right = right * 0.5 * ParticleSize;
	direction = direction * 0.5 * ParticleSize;


	// a: left-bottom
	vec3 va = P.xyz - right - direction;
	gl_Position = ProjectionMatrix * vec4(va, P.w);
	TexCoord0 = vec2(0.0, 0.0);
	Velocity0 = vertex[0].Velocity1;
	TTL0 = vertex[0].TTL1;
	Type0 = vertex[0].Type1;
	EmitVertex();

	// b: right-bottom
	vec3 vb = P.xyz + right - direction;
	gl_Position = ProjectionMatrix * vec4(vb, P.w);
	TexCoord0 = vec2(1.0, 0.0);
	Velocity0 = vertex[0].Velocity1;
	TTL0 = vertex[0].TTL1;
	Type0 = vertex[0].Type1;
	EmitVertex();


	// c: left-top
	vec3 vc = P.xyz - right + direction;
	gl_Position = ProjectionMatrix * vec4(vc, P.w);
	TexCoord0 = vec2(0.0, 1.0);
	Velocity0 = vertex[0].Velocity1;
	TTL0 = vertex[0].TTL1;
	Type0 = vertex[0].Type1;
	EmitVertex();

	// d: right-top
	vec3 vd = P.xyz + right + direction;
	gl_Position = ProjectionMatrix * vec4(vd, P.w);
	TexCoord0 = vec2(1.0, 1.0);
	Velocity0 = vertex[0].Velocity1;
	TTL0 = vertex[0].TTL1;
	Type0 = vertex[0].Type1;
	EmitVertex();
}
*/

// standard facing quad -> orientation is dependent on camera rotation
// = view plane alignment
void emitViewPlaneFacingQuad(vec2 size)
{
	vec4 P = gl_in[0].gl_Position;

	// a: left-bottom 
	vec2 va = P.xy + vec2(-0.5, -0.5) * size;
	gl_Position = ProjectionMatrix * vec4(va, P.zw);
	TexCoord0 = vec2(0.0, 0.0);
	Velocity0 = vertex[0].Velocity1;
	TTL0 = vertex[0].TTL1;
	Type0 = vertex[0].Type1;
	Color0 = vertex[0].Color1;
	EmitVertex();
  
	// b: right-bottom
	vec2 vd = P.xy + vec2(0.5, -0.5) * size;
	gl_Position = ProjectionMatrix * vec4(vd, P.zw);
	TexCoord0 = vec2(1.0, 0.0);
	Velocity0 = vertex[0].Velocity1;
	TTL0 = vertex[0].TTL1;
	Type0 = vertex[0].Type1;
	Color0 = vertex[0].Color1;
	EmitVertex();

	// c: left-top
	vec2 vb = P.xy + vec2(-0.5, 0.5) * size;
	gl_Position = ProjectionMatrix * vec4(vb, P.zw);
	TexCoord0 = vec2(0.0, 1.0);
	Velocity0 = vertex[0].Velocity1;
	TTL0 = vertex[0].TTL1;
	Type0 = vertex[0].Type1;
	Color0 = vertex[0].Color1;
	EmitVertex();
  
	// d: right-top
	vec2 vc = P.xy + vec2(0.5, 0.5) * size;
	gl_Position = ProjectionMatrix * vec4(vc, P.zw);
	TexCoord0 = vec2(1.0, 1.0);
	Velocity0 = vertex[0].Velocity1;
	TTL0 = vertex[0].TTL1;
	Type0 = vertex[0].Type1;
	Color0 = vertex[0].Color1;
	EmitVertex();

	EndPrimitive();
}

// quad faces the camera location and is aligned along the given up vector
// = axial billboard
void emitViewPointFacingQuad(vec2 size, vec3 up)
{
	vec4 P = gl_in[0].gl_Position;
	vec3 P3 = P.xyz;

	up = normalize(ModelViewMatrix * vec4(up, 0)).xyz; // up vector in world space -> to view space
	vec3 direction = normalize(P3); // from camera to point
	vec3 right = cross(direction, up);

	right = right * 0.5 * size.x;
	up = up * 0.5 * size.y;


	// a: left-bottom
	vec3 va = P3 - right - up;
	gl_Position = ProjectionMatrix * vec4(va, P.w);
	TexCoord0 = vec2(0.0, 0.0);
	Velocity0 = vertex[0].Velocity1;
	TTL0 = vertex[0].TTL1;
	Type0 = vertex[0].Type1;
	Color0 = vertex[0].Color1;
	EmitVertex();

	// b: right-bottom
	vec3 vb = P3 + right - up;
	gl_Position = ProjectionMatrix * vec4(vb, P.w);
	TexCoord0 = vec2(1.0, 0.0);
	Velocity0 = vertex[0].Velocity1;
	TTL0 = vertex[0].TTL1;
	Type0 = vertex[0].Type1;
	Color0 = vertex[0].Color1;
	EmitVertex();


	// c: left-top
	vec3 vc = P3 - right + up;
	gl_Position = ProjectionMatrix * vec4(vc, P.w);
	TexCoord0 = vec2(0.0, 1.0);
	Velocity0 = vertex[0].Velocity1;
	TTL0 = vertex[0].TTL1;
	Type0 = vertex[0].Type1;
	Color0 = vertex[0].Color1;
	EmitVertex();

	// d: right-top
	vec3 vd = P3 + right + up;
	gl_Position = ProjectionMatrix * vec4(vd, P.w);
	TexCoord0 = vec2(1.0, 1.0);
	Velocity0 = vertex[0].Velocity1;
	TTL0 = vertex[0].TTL1;
	Type0 = vertex[0].Type1;
	Color0 = vertex[0].Color1;
	EmitVertex();
}

void main (void)
{
	float particleStartTime = vertex[0].Velocity1.w;
	if (CurrentTime < particleStartTime) return; // don't draw particles with a future start time

	uint type = vertex[0].Type1;
	bool isTrail = type >= 2u && type <= 5u;

	if (isTrail)
	{
		vec3 up = vertex[0].Velocity1.xyz;
		const vec2 size = vec2(60, 270) * 0.006;
		vec2 size2 = (ModelMatrix * vec4(size, 0, 0)).xy;
		emitViewPointFacingQuad(size2, up);
	}
	else if (type == 0u)
	{
		// start particle -> round
		const vec2 size = vec2(5, 5);
		vec2 size2 = (ModelMatrix * vec4(size, 0, 0)).xy;
		emitViewPlaneFacingQuad(size2); // round particles should always face the camera
	}
	else if (type == 1u)
	{
		// explosion particle
		const vec2 size = vec2(250, 250) * 0.05;
		vec2 size2 = (ModelMatrix * vec4(size, 0, 0)).xy;
		emitViewPlaneFacingQuad(size2);
	}
}
