#version 330

#include "util/vertexSkinning.glsl"
#line 5

layout (location = 0) in vec3 Position;
layout (location = 1) in vec3 Normal;
layout (location = 2) in vec2 TexCoord;
layout (location = 4) in vec3 Tangent;
layout (location = 5) in vec3 Bitangent;

uniform mat4 MvpMatrix;
uniform mat4 ModelMatrix;
uniform vec4 ClipPlane;

out vec2 TexCoord0;
out vec3 Normal0;
out vec3 Tangent0;

void main()
{
	//clip-plane to split the scene----------------------------------------------
	//	https://www.opengl.org/sdk/docs/man/html/gl_ClipDistance.xhtml
	gl_ClipDistance[0] = dot(ModelMatrix *vec4(Position, 1.0f), ClipPlane);
	//---------------------------------------------------------------------------

	mat4 animation = animationTransformation();

	gl_Position = MvpMatrix * animation * vec4(Position, 1.0);
	TexCoord0 = TexCoord;

	vec3 N = Normal;
	vec3 T = Tangent;
	vec3 B = Bitangent;

	// adapt the tangent space according to its handedness
	// this is necessary for mirrored geometry, using the same UV coordinates
	// e.g. for the dragon (there would be a visible seam when applying the normal map, without correction)
	vec3 bt = cross(N, T);
	float orientation = step(0, dot(bt, B)) * 2 - 1; // is -1 if the tangent space is left-handed (i.e. if the bitangents are not pointing in the same direction)
	T.x *= orientation; // I'm pretty sure that's model specific, but it works for us

	// TODO use normal matrix instead:
	Normal0 = (ModelMatrix * animation * vec4(N, 0.0)).xyz;
	Tangent0 = (ModelMatrix * animation * vec4(T, 0.0)).xyz;
}
