#version 330
#pragma optionNV (unroll all)

in vec2 TexCoord0;

out vec4 FragColor;

uniform sampler2D NormalTexture; // normal texture from GBuffer
uniform sampler2D Texture0; // texture to blur

uniform vec2 Direction; // direction of blur. Typically either (0, 1) or (1, 0)

const float indices[9] = float[] (-4, -3, -2, -1, 0, +1, +2, +3, +4);

void main()
{
	ivec2 texSizeNormal = textureSize(NormalTexture, 0); // size of normal texture
	ivec2 texSizeToBlur = textureSize(Texture0, 0); // size of texture to blur

	vec2 offsetDir1 = Direction / texSizeNormal;
	vec2 offsetDir2 = Direction / texSizeToBlur;

	vec3 normal[9];
	normal[0] = texture(NormalTexture, TexCoord0 + indices[0] * offsetDir1).xyz;
	normal[1] = texture(NormalTexture, TexCoord0 + indices[1] * offsetDir1).xyz;
	normal[2] = texture(NormalTexture, TexCoord0 + indices[2] * offsetDir1).xyz;
	normal[3] = texture(NormalTexture, TexCoord0 + indices[3] * offsetDir1).xyz;
	normal[4] = texture(NormalTexture, TexCoord0 + indices[4] * offsetDir1).xyz;
	normal[5] = texture(NormalTexture, TexCoord0 + indices[5] * offsetDir1).xyz;
	normal[6] = texture(NormalTexture, TexCoord0 + indices[6] * offsetDir1).xyz;
	normal[7] = texture(NormalTexture, TexCoord0 + indices[7] * offsetDir1).xyz;
	normal[8] = texture(NormalTexture, TexCoord0 + indices[8] * offsetDir1).xyz;

	// Matlab: fspecial('gaussian', [9 1], 1.67);
	float weights[9] = float[] (
		0.0136499001638555,
		0.0478804503945193,
		0.117344970059215,
		0.200931593927748,
		0.240386170909323,
		0.200931593927748,
		0.117344970059215,
		0.0478804503945193,
		0.0136499001638555
	);

	float totalWeight = 1.0; // sum of all weights is 1 of course
	float discardThreshold = 0.85;

	int i;

	for( i=0; i<9; ++i )
	{
		if( dot(normal[i], normal[4]) < discardThreshold )
		{
			totalWeight -= weights[i];
			weights[i] = 0;
		}
	}

	vec4 result = vec4(0);
	for( i=0; i<9; ++i )
	{
		result += texture(Texture0, TexCoord0 + indices[i] * offsetDir2) * weights[i];
	}

	result /= totalWeight;
	FragColor = result;
}
