
/*
vec3 xyz2rgb_V2(vec3 xyz)
{
	// http://content.gpwiki.org/index.php/D3DBook:High-Dynamic_Range_Rendering

	// column major!
	const mat3 m = mat3(
						vec3(2.5651, -1.0217, 0.0753),
						vec3(-1.1665, 1.9777, -0.2543),
						vec3(-0.3986, 0.0439, 1.1892)
						);
	
	return m * xyz;
}

vec3 rgb2xyz_V2(vec3 rgb)
{
	// http://content.gpwiki.org/index.php/D3DBook:High-Dynamic_Range_Rendering

	// column major!
	const mat3 m = mat3(
						vec3(0.5141364, 0.265068, 0.0241188),
						vec3(0.3238786, 0.67023428, 0.1228178),
						vec3(0.16036376, 0.06409157, 0.84442666)
						);

	return m * rgb;
}
*/

vec3 xyz2rgb_V1(vec3 xyz)
{
	// http://www.brucelindbloom.com/Eqn_RGB_XYZ_Matrix.html
	// XYZ -> sRGB with D65 white point

	// column major!
	const mat3 m = mat3(
						vec3(3.2404542, -0.9692660, 0.0556434),
						vec3(-1.5371385, 1.8760108, -0.2040259),
						vec3(-0.4985314, 0.0415560, 1.0572252)
						);
	
	return m * xyz;
}

vec3 rgb2xyz_V1(vec3 rgb)
{
	// http://www.brucelindbloom.com/Eqn_RGB_XYZ_Matrix.html
	// sRGB -> XYZ with D65 white point

	// column major!
	const mat3 m = mat3(
						vec3(0.4124564, 0.2126729, 0.0193339),
						vec3(0.3575761, 0.7151522, 0.1191920),
						vec3(0.1804375, 0.0721750, 0.9503041)
						);

	return m * rgb;
}

vec3 rgb2Yxy(vec3 rgb) 
{
	vec3 XYZ = rgb2xyz_V1(rgb);

	// XYZ -> Yxy conversion
	vec3 Yxy;
	float sum = dot(XYZ, vec3(1));
	
	Yxy.r = XYZ.g;			// Y = Y
	Yxy.g = XYZ.r / sum;	// x = X / (X + Y + Z)
	Yxy.b = XYZ.g / sum;	// y = Y / (X + Y + Z)

	return Yxy;
}

vec3 Yxy2rgb(vec3 Yxy) 
{
	vec3 XYZ;

	// Yxy -> XYZ conversion
	XYZ.r = Yxy.r * Yxy.g / Yxy. b;					// X = Y * x / y
	XYZ.g = Yxy.r;									// Y = Y
	XYZ.b = Yxy.r * (1 - Yxy.g - Yxy.b) / Yxy.b;	// Z = Y * (1-x-y) / y

	vec3 rgb = xyz2rgb_V1(XYZ);
	return rgb;
}
