#version 330 core

// adapted from http://http.developer.nvidia.com/GPUGems2/gpugems2_chapter19.html

in vec4 Wave0;
in vec2 Wave1;
in vec2 Wave2;
in vec2 Wave3;

in vec4 ScreenPos;
in vec3 Eye;
in vec3 FragPos;
in vec3 Normal;

uniform sampler2D textNormal;			//Normal Map
uniform sampler2D textRefraction;		//Refraction map
uniform sampler2D textReflection;		//Reflection map

uniform vec3 WaterColor;

out vec4 color;

float Fresnel(float NdotL, float fresnelBias, float fresnelPow);

void main()
{
	vec3 vEye = normalize(Eye);

	// Get bump layers, normalComponent = 2 * (colorComponent - 0.5);
	vec3 vBumpTexA = texture2D(textNormal, Wave0.xy).xyz;
	vec3 vBumpTexB = texture2D(textNormal, Wave1.xy).xyz;
	vec3 vBumpTexC = texture2D(textNormal, Wave2.xy).xyz;
	vec3 vBumpTexD = texture2D(textNormal, Wave3.xy).xyz;

	// Average bump layers
	vec3 vBumpTex = normalize(2.0 * (vBumpTexA.xyz + vBumpTexB.xyz + vBumpTexC.xyz + vBumpTexD.xyz) - 4.0);

	// Apply individual bump scale for refraction and reflection
	vec3 vRefrBump = vBumpTex * vec3(0.04, 0.04, 1.0);
	vec3 vReflBump = vBumpTex * vec3(0.05, 0.05, 1.0);

	// Compute projected coordinates
	vec2 vProj = ScreenPos.xy / ScreenPos.w;


	vec2 coords = vProj + vReflBump.xy;
	vec2 coords2 = vProj + vRefrBump.xy;
	vec3 vReflection = texture2D(textReflection, coords).rgb;
	vec4 vRefrA = texture2D(textRefraction, vec2(coords2.x, 1 - coords2.y));

	// Mask occluders from refraction map

	// What are you doing here?
	//vec4 vRefrB = texture2D(textRefraction, vec2(vProj.x, 1 - vProj.y));
	//vec3 vRefraction = (vRefrB * vRefrA.w + vRefrA * (1 - vRefrA.w)).rgb;
	
	vec3 vRefraction = vRefrA.xyz;

	// Compute Fresnel term
	float NdotL = max(dot(vEye, normalize(vReflBump)), 0.0);
	float facing = (1.0 - NdotL);
	//const float R0 = 0.02037; // Schlick's approximation R0 for water/air, n1 = 1, n2 = 1.333
	const float R0 = 0.1; // higher constant, because otherwise the reflections would be barely visible
	float fresnel = Fresnel(NdotL, R0, 5.0);


	// Use distance to lerp between refraction and deep water color
	float dist = Wave0.w;// effects deep water color
	dist = min(dist, 30);
	float fDistScale = clamp(20.0 / dist, 0.0, 1.0);
	vec3 WaterDeepColor = (vRefraction * fDistScale + (1 - fDistScale) * WaterColor);


	// Lerp between water color and deep water color
	vec3 waterColor = (WaterColor * facing + WaterDeepColor * (1.0 - facing));
	vec3 cReflect = fresnel * vReflection;


	// final water = reflection_color * fresnel + water_color
	color = vec4(cReflect + waterColor * (1 - fresnel), 1.0);

	//color = vec4(vRefraction, 1.0f);
}

float Fresnel(float NdotL, float fresnelBias, float fresnelPow)
{
	// https://en.wikipedia.org/wiki/Schlick's_approximation
	float facing = (1.0 - NdotL);
	return max(fresnelBias + (1.0 - fresnelBias) * pow(facing, fresnelPow), 0.0);
}

